@@ -1,4 +1,4 @@
-#!/usr/bin/env python2
+#!/usr/bin/env python
 #
 # Electrum - lightweight Bitcoin client
 # Copyright (C) 2012 thomasv@gitorious
@@ -23,38 +23,44 @@
 # CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 # SOFTWARE.
 
-import sys
-import os
 import signal
+import sys
+import traceback
+
 
 try:
-    import PyQt4
+    import PyQt5
 except Exception:
-    sys.exit("Error: Could not import PyQt4 on Linux systems, you may try 'sudo apt-get install python-qt4'")
+    sys.exit("Error: Could not import PyQt5 on Linux systems, you may try 'sudo apt-get install python3-pyqt5'")
 
-from PyQt4.QtGui import *
-from PyQt4.QtCore import *
-import PyQt4.QtCore as QtCore
+from PyQt5.QtGui import *
+from PyQt5.QtWidgets import *
+from PyQt5.QtCore import *
+import PyQt5.QtCore as QtCore
 
 from electrum.i18n import _, set_language
 from electrum.plugins import run_hook
-from electrum import SimpleConfig, Wallet, WalletStorage
-from electrum.synchronizer import Synchronizer
-from electrum.verifier import SPV
-from electrum.util import DebugMem, UserCancelled, InvalidPassword
-from electrum.wallet import Abstract_Wallet
-from installwizard import InstallWizard, GoBack
+from electrum import WalletStorage
+# from electrum.synchronizer import Synchronizer
+# from electrum.verifier import SPV
+# from electrum.util import DebugMem
+from electrum.util import UserCancelled, print_error
+# from electrum.wallet import Abstract_Wallet
+
+from .installwizard import InstallWizard, GoBack
 
 
 try:
-    import icons_rc
-except Exception:
-    print "Error: Could not find icons file."
-    print "Please run 'pyrcc4 icons.qrc -o gui/qt/icons_rc.py', and reinstall Electrum"
+    from . import icons_rc
+except Exception as e:
+    print(e)
+    print("Error: Could not find icons file.")
+    print("Please run 'pyrcc5 icons.qrc -o gui/qt/icons_rc.py', and reinstall Electrum")
     sys.exit(1)
 
-from util import *   # * needed for plugins
-from main_window import ElectrumWindow
+from .util import *   # * needed for plugins
+from .main_window import ElectrumWindow
+from .network_dialog import NetworkDialog
 
 
 class OpenFileEventFilter(QObject):
@@ -70,6 +76,13 @@ class OpenFileEventFilter(QObject):
         return False
 
 
+class QElectrumApplication(QApplication):
+    new_window_signal = pyqtSignal(str, object)
+
+
+class QNetworkUpdatedSignalObject(QObject):
+    network_updated_signal = pyqtSignal(str, object)
+
 
 class ElectrumGui:
 
@@ -79,15 +92,21 @@ class ElectrumGui:
         # GC-ed when windows are closed
         #network.add_jobs([DebugMem([Abstract_Wallet, SPV, Synchronizer,
         #                            ElectrumWindow], interval=5)])
+        QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_X11InitThreads)
+        if hasattr(QtCore.Qt, "AA_ShareOpenGLContexts"):
+            QtCore.QCoreApplication.setAttribute(QtCore.Qt.AA_ShareOpenGLContexts)
+        if hasattr(QGuiApplication, 'setDesktopFileName'):
+            QGuiApplication.setDesktopFileName('electrum.desktop')
         self.config = config
         self.daemon = daemon
         self.plugins = plugins
         self.windows = []
         self.efilter = OpenFileEventFilter(self.windows)
-        self.app = QApplication(sys.argv)
+        self.app = QElectrumApplication(sys.argv)
         self.app.installEventFilter(self.efilter)
         self.timer = Timer()
         self.nd = None
+        self.network_updated_signal_obj = QNetworkUpdatedSignalObject()
         # init tray
         self.dark_icon = self.config.get("dark_icon", False)
         self.tray = QSystemTrayIcon(self.tray_icon(), None)
@@ -95,13 +114,18 @@ class ElectrumGui:
         self.tray.activated.connect(self.tray_activated)
         self.build_tray_menu()
         self.tray.show()
-        self.app.connect(self.app, QtCore.SIGNAL('new_window'), self.start_new_window)
+        self.app.new_window_signal.connect(self.start_new_window)
         run_hook('init_qt', self)
+        ColorScheme.update_from_widget(QWidget())
 
     def build_tray_menu(self):
         # Avoid immediate GC of old menu when window closed via its action
-        self.old_menu = self.tray.contextMenu()
-        m = QMenu()
+        if self.tray.contextMenu() is None:
+            m = QMenu()
+            self.tray.setContextMenu(m)
+        else:
+            m = self.tray.contextMenu()
+            m.clear()
         for window in self.windows:
             submenu = m.addMenu(window.wallet.basename())
             submenu.addAction(_("Show/Hide"), window.show_or_hide)
@@ -109,7 +133,6 @@ class ElectrumGui:
         m.addAction(_("Dark/Light"), self.toggle_tray_icon)
         m.addSeparator()
         m.addAction(_("Exit Electrum"), self.close)
-        self.tray.setContextMenu(m)
 
     def tray_icon(self):
         if self.dark_icon:
@@ -137,10 +160,9 @@ class ElectrumGui:
 
     def new_window(self, path, uri=None):
         # Use a signal as can be called from daemon thread
-        self.app.emit(SIGNAL('new_window'), path, uri)
+        self.app.new_window_signal.emit(path, uri)
 
     def show_network_dialog(self, parent):
-        from network_dialog import NetworkDialog
         if not self.daemon.network:
             parent.show_warning(_('You are using Electrum in offline mode; restart Electrum if you want to get connected'), title=_('Offline'))
             return
@@ -149,7 +171,8 @@ class ElectrumGui:
             self.nd.show()
             self.nd.raise_()
             return
-        self.nd = NetworkDialog(self.daemon.network, self.config)
+        self.nd = NetworkDialog(self.daemon.network, self.config,
+                                self.network_updated_signal_obj)
         self.nd.show()
 
     def create_window_for_wallet(self, wallet):
@@ -168,19 +191,43 @@ class ElectrumGui:
                 w.bring_to_top()
                 break
         else:
-            wallet = self.daemon.load_wallet(path, None)
+            try:
+                wallet = self.daemon.load_wallet(path, None)
+            except BaseException as e:
+                traceback.print_exc(file=sys.stdout)
+                d = QMessageBox(QMessageBox.Warning, _('Error'),
+                                _('Cannot load wallet:') + '\n' + str(e))
+                d.exec_()
+                return
             if not wallet:
-                storage = WalletStorage(path)
+                storage = WalletStorage(path, manual_upgrades=True)
                 wizard = InstallWizard(self.config, self.app, self.plugins, storage)
-                wallet = wizard.run_and_get_wallet()
+                try:
+                    wallet = wizard.run_and_get_wallet()
+                except UserCancelled:
+                    pass
+                except GoBack as e:
+                    print_error('[start_new_window] Exception caught (GoBack)', e)
                 wizard.terminate()
                 if not wallet:
                     return
                 wallet.start_threads(self.daemon.network)
                 self.daemon.add_wallet(wallet)
-            w = self.create_window_for_wallet(wallet)
+            try:
+                w = self.create_window_for_wallet(wallet)
+            except BaseException as e:
+                traceback.print_exc(file=sys.stdout)
+                d = QMessageBox(QMessageBox.Warning, _('Error'),
+                                _('Cannot create window for wallet:') + '\n' + str(e))
+                d.exec_()
+                return
         if uri:
             w.pay_to_URI(uri)
+        w.bring_to_top()
+        w.setWindowState(w.windowState() & ~QtCore.Qt.WindowMinimized | QtCore.Qt.WindowActive)
+
+        # this will activate the window
+        w.activateWindow()
         return w
 
     def close_window(self, window):
@@ -206,7 +253,7 @@ class ElectrumGui:
             return
         except GoBack:
             return
-        except:
+        except BaseException as e:
             traceback.print_exc(file=sys.stdout)
             return
         self.timer.start()
@@ -215,11 +262,23 @@ class ElectrumGui:
         if not self.start_new_window(path, self.config.get('url')):
             return
         signal.signal(signal.SIGINT, lambda *args: self.app.quit())
+
+        def quit_after_last_window():
+            # on some platforms, not only does exec_ not return but not even
+            # aboutToQuit is emitted (but following this, it should be emitted)
+            if self.app.quitOnLastWindowClosed():
+                self.app.quit()
+        self.app.lastWindowClosed.connect(quit_after_last_window)
+
+        def clean_up():
+            # Shut down the timer cleanly
+            self.timer.stop()
+            # clipboard persistence. see http://www.mail-archive.com/pyqt@riverbankcomputing.com/msg17328.html
+            event = QtCore.QEvent(QtCore.QEvent.Clipboard)
+            self.app.sendEvent(self.app.clipboard(), event)
+            self.tray.hide()
+        self.app.aboutToQuit.connect(clean_up)
+
         # main loop
         self.app.exec_()
-        # Shut down the timer cleanly
-        self.timer.stop()
-        # clipboard persistence. see http://www.mail-archive.com/pyqt@riverbankcomputing.com/msg17328.html
-        event = QtCore.QEvent(QtCore.QEvent.Clipboard)
-        self.app.sendEvent(self.app.clipboard(), event)
-        self.tray.hide()
+        # on some platforms the exec_ call may not return, so use clean_up()
